@@ -29,6 +29,7 @@
 #  include <libbonoboui.h>
 #  include <libgnomevfs/gnome-vfs.h>
 #  include <gtk/gtk.h>
+#  include "gpdf-control-private.h"
 #include "gpdf-g-switch.h"
 #include "Object.h"
 #include "ErrorCodes.h"
@@ -134,8 +135,19 @@ gpdf_persist_file_on_okbutton_clicked (GtkButton *button,
     gtk_main_quit();
 }
 
+static void
+gpdf_persist_file_on_helpbutton_clicked (GtkButton *button,
+					 gpointer   user_data)
+{
+    GtkWidget *dialog1 = GTK_WIDGET(user_data);
+    BonoboControl *control =
+      BONOBO_CONTROL (g_object_get_data (G_OBJECT (dialog1), "control"));
+    gpdf_control_private_display_help (control, "gpdf-enter-password");
+}
+
 static GtkWidget *
-gpdf_persist_file_create_password_dialog (const gchar *labeltext,
+gpdf_persist_file_create_password_dialog (GPdfPersistFile *pf,
+					  const gchar *labeltext,
 					  const gchar *tiptext)
 {
     GtkWidget *dialog1;
@@ -146,10 +158,13 @@ gpdf_persist_file_create_password_dialog (const gchar *labeltext,
     GtkWidget *label;
     GtkWidget *entry1;
     GtkWidget *dialog_action_area1;
+    GtkWidget *helpbutton1;
     GtkWidget *cancelbutton1;
     GtkWidget *okbutton1;
     GtkTooltips *tooltips;
 
+    g_return_val_if_fail (GPDF_IS_PERSIST_FILE (pf), NULL);
+    
     tooltips = gtk_tooltips_new ();
     
     dialog1 = gtk_widget_new (GTK_TYPE_DIALOG,
@@ -158,6 +173,7 @@ gpdf_persist_file_create_password_dialog (const gchar *labeltext,
 			      "resizable", FALSE,
 			      "has-separator", FALSE,
 			      NULL);
+    g_object_set_data (G_OBJECT (dialog1), "control", pf->priv->control);
     
     dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
     g_object_set (dialog_vbox1, "spacing", 12, NULL);
@@ -172,7 +188,6 @@ gpdf_persist_file_create_password_dialog (const gchar *labeltext,
     gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);
 
     image = gtk_widget_new (GTK_TYPE_IMAGE,
-			    /* FIXME use gtk stock icon from gtk 2.3 */
 			    "stock", GTK_STOCK_DIALOG_AUTHENTICATION,
 			    "icon-size", GTK_ICON_SIZE_DIALOG,
 			    "yalign", 0.0,
@@ -205,6 +220,12 @@ gpdf_persist_file_create_password_dialog (const gchar *labeltext,
     gtk_widget_show (dialog_action_area1);
     gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
 
+    helpbutton1 = gtk_button_new_from_stock ("gtk-help");
+    gtk_widget_set_name (helpbutton1, "helpbutton1");
+    gtk_widget_show (helpbutton1);
+    gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), helpbutton1, GTK_RESPONSE_HELP);
+    GTK_WIDGET_SET_FLAGS (helpbutton1, GTK_CAN_DEFAULT);
+
     cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
     gtk_widget_show (cancelbutton1);
     gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), cancelbutton1, GTK_RESPONSE_CANCEL);
@@ -224,55 +245,38 @@ gpdf_persist_file_create_password_dialog (const gchar *labeltext,
     g_signal_connect ((gpointer) okbutton1, "clicked",
 		      G_CALLBACK (gpdf_persist_file_on_okbutton_clicked),
 		      (gpointer)dialog1);
+    g_signal_connect ((gpointer) helpbutton1, "clicked",
+		      G_CALLBACK (gpdf_persist_file_on_helpbutton_clicked),
+		      (gpointer)dialog1);
     return dialog1;
 }
 
 static gchar *
-gpdf_persist_file_get_password (GPdfPersistFile *pf,
-				const gchar *labeltext,
-				const gchar *tiptext)
+gpdf_persist_file_get_password (GPdfPersistFile *pf)
 {
-    GPdfPersistFilePrivate *priv;
-    gchar *passwd;
-    GtkWidget *dialog;
+	BonoboControl *control;
+	gchar *passwd;
+	GtkWidget *dialog;
+	
+	g_return_val_if_fail (GPDF_IS_PERSIST_FILE (pf), NULL);
+	
+	control = BONOBO_CONTROL (pf->priv->control);
 
-    g_return_val_if_fail (GPDF_IS_PERSIST_FILE (pf), NULL);
-
-    priv = pf->priv;
-
-    dialog = gpdf_persist_file_create_password_dialog (labeltext,
-						       tiptext);
-    
-    bonobo_control_set_transient_for(BONOBO_CONTROL (priv->control),
-				     GTK_WINDOW (dialog),
-				     NULL);
-    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-    gtk_widget_show (dialog);
-    gtk_main();
-    passwd = g_strdup ((const gchar *)g_object_get_data (G_OBJECT (dialog),
-							 "passwd"));
-    gtk_widget_destroy (dialog);
-    return passwd;
-}
-
-static gchar *
-gpdf_persist_file_get_owner_password (GPdfPersistFile *pf)
-{
-    return gpdf_persist_file_get_password
-      (pf,
-       _("Enter document owner password:"),
-       _("This document is password protected and this "
-	 "operation requires the document's owner password"));
-}
-
-static gchar *
-gpdf_persist_file_get_user_password (GPdfPersistFile *pf)
-{
-    return gpdf_persist_file_get_password
-      (pf,
-       _("Enter document user password:"),
-       _("This document is password protected and this "
-	 "operation requires the document's user password"));
+	dialog = gpdf_persist_file_create_password_dialog (
+		pf, 
+		_("Enter document password:"),
+		_("This document is encrypted and this "
+		  "operation requires the document's password"));
+	
+	bonobo_control_set_transient_for(control,
+					 GTK_WINDOW (dialog),
+					 NULL);
+	gtk_widget_show (dialog);
+	gtk_main();
+	passwd = g_strdup ((const gchar *)g_object_get_data (G_OBJECT (dialog),
+							     "passwd"));
+	gtk_widget_destroy (dialog);
+	return passwd;
 }
 
 static void
@@ -287,8 +291,6 @@ impl_bonobo_persist_file_load (PortableServer_Servant servant,
 	GnomeVFSResult vfs_result;
 	Object obj;
 	BaseStream *pdf_stream;
-	GString user_pwd, owner_pwd;
-	int err;
 	const gchar *error_msg;
 
 	bonobo_return_if_fail (GPDF_IS_NON_NULL_PERSIST_FILE (pf), ev);
@@ -324,17 +326,16 @@ impl_bonobo_persist_file_load (PortableServer_Servant servant,
 
 	/* FIXME if (pdf_doc == NULL) ? */
 	if (!priv->pdf_doc->isOk ()) {
+		int err;
 		err = priv->pdf_doc->getErrorCode ();
+
 		if (err == errEncrypted) {
-			user_pwd.append(gpdf_persist_file_get_user_password (pf));
-			priv->pdf_doc = new PDFDoc (pdf_stream, &user_pwd, &user_pwd);
-			if (!priv->pdf_doc->isOk ()) {
-				err = priv->pdf_doc->getErrorCode ();
-				if (err == errEncrypted) {
-					owner_pwd.append(gpdf_persist_file_get_owner_password (pf));
-					priv->pdf_doc = new PDFDoc (pdf_stream, &user_pwd, &owner_pwd);
-				}
-			}
+			gchar *pwd = gpdf_persist_file_get_password (pf);
+			GString goo_pwd (pwd);
+			g_free (pwd);
+
+			/* delete priv->pdf_doc; */ /* FIXME causes SEGV */
+			priv->pdf_doc = new PDFDoc (pdf_stream, &goo_pwd, &goo_pwd);
 		}
 	}
 	if (priv->pdf_doc->isOk ()) {
@@ -356,7 +357,7 @@ impl_bonobo_persist_file_load (PortableServer_Servant servant,
 		bonobo_exception_set (ev, ex_Bonobo_IOError);	
 		break;
 	case errEncrypted:
-		error_msg = _("The PDF document is encrypted but you didn't give the right password");
+		error_msg = _("The PDF document is encrypted and you didn't enter the correct password");
 		bonobo_exception_set (ev, ex_Bonobo_IOError);	
 		break;
 	case errNone:
